@@ -93,6 +93,11 @@ typedef struct {
 	GtkWidget         *label_topic;
 	GtkWidget         *contact_list_view;
 	GtkWidget         *info_bar_vbox;
+
+	guint              unread_messages;
+	/* TRUE if the pending messages can be displayed. This is to avoid to show
+	 * pending messages *before* messages from logs. (#603980) */
+	gboolean           can_show_pending;
 } EmpathyChatPriv;
 
 typedef struct {
@@ -203,6 +208,43 @@ chat_connect_channel_reconnected (EmpathyDispatchOperation *dispatch,
 	}
 }
 
+static void
+reconnected_connection_ready_cb (TpConnection *connection,
+			const GError *error,
+			gpointer user_data)
+{
+	EmpathyChat *chat = user_data;
+	EmpathyChatPriv *priv = GET_PRIV (chat);
+
+	if (error != NULL) {
+		DEBUG ("connection is not ready: %s", error->message);
+		goto out;
+	}
+
+	DEBUG ("Account reconnected, request a new Text channel");
+
+	switch (priv->handle_type) {
+		case TP_HANDLE_TYPE_CONTACT:
+			empathy_dispatcher_chat_with_contact_id (
+				connection, priv->id,
+				chat_connect_channel_reconnected,
+				chat);
+			break;
+		case TP_HANDLE_TYPE_ROOM:
+			empathy_dispatcher_join_muc (connection,
+				priv->id,
+				chat_connect_channel_reconnected,
+				chat);
+			break;
+		default:
+			g_assert_not_reached ();
+			break;
+	}
+
+out:
+	g_object_unref (chat);
+}
+
 static void
 chat_new_connection_cb (TpAccount   *account,
 			guint        old_status,
@@ -215,32 +257,19 @@ chat_new_connection_cb (TpAccount   *account,
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	TpConnection *connection;
 
+	if (new_status != TP_CONNECTION_STATUS_CONNECTED)
+		return;
+
 	connection = tp_account_get_connection (account);
 
-	if (!priv->tp_chat && account == priv->account &&
-	    priv->handle_type != TP_HANDLE_TYPE_NONE &&
-	    !EMP_STR_EMPTY (priv->id)) {
+	if (priv->tp_chat != NULL || account != priv->account ||
+	    priv->handle_type == TP_HANDLE_TYPE_NONE ||
+	    EMP_STR_EMPTY (priv->id))
+		return;
 
-		DEBUG ("Account reconnected, request a new Text channel");
-
-		switch (priv->handle_type) {
-			case TP_HANDLE_TYPE_CONTACT:
-				empathy_dispatcher_chat_with_contact_id (
-					connection, priv->id,
-					chat_connect_channel_reconnected,
-					chat);
-				break;
-			case TP_HANDLE_TYPE_ROOM:
-				empathy_dispatcher_join_muc (connection,
-					priv->id,
-					chat_connect_channel_reconnected,
-					chat);
-				break;
-			default:
-				g_assert_not_reached ();
-				break;
-		}
-	}
+	g_object_ref (chat);
+	tp_connection_call_when_ready (connection, reconnected_connection_ready_cb,
+				   chat);
 }
 
 static void
@@ -1074,6 +1103,7 @@ chat_message_received (EmpathyChat *chat, EmpathyMessage *message)
 			       TP_CHANNEL_CHAT_STATE_ACTIVE,
 			       chat);
 
+	priv->unread_messages++;
 	g_signal_emit (chat, signals[NEW_MESSAGE], 0, message);
 }
 
@@ -1795,6 +1825,8 @@ chat_members_changed_cb (EmpathyTpChat  *tp_chat,
 	const gchar *name = empathy_contact_get_name (contact);
 	gchar *str;
 
+	g_return_if_fail (TP_CHANNEL_GROUP_CHANGE_REASON_RENAMED != reason);
+
 	if (priv->block_events_timeout_id != 0)
 		return;
 
@@ -1809,6 +1841,30 @@ chat_members_changed_cb (EmpathyTpChat  *tp_chat,
 	g_free (str);
 }
 
+static void
+chat_member_renamed_cb (EmpathyTpChat  *tp_chat,
+			 EmpathyContact *old_contact,
+			 EmpathyContact *new_contact,
+			 guint           reason,
+			 gchar          *message,
+			 EmpathyChat    *chat)
+{
+	EmpathyChatPriv *priv = GET_PRIV (chat);
+
+	g_return_if_fail (TP_CHANNEL_GROUP_CHANGE_REASON_RENAMED == reason);
+
+	if (priv->block_events_timeout_id == 0) {
+		gchar *str;
+
+		str = g_strdup_printf (_("%s is now known as %s"),
+				       empathy_contact_get_name (old_contact),
+				       empathy_contact_get_name (new_contact));
+		empathy_chat_view_append_event (chat->view, str);
+		g_free (str);
+	}
+
+}
+
 static gboolean
 chat_reset_size_request (gpointer widget)
 {
@@ -1818,17 +1874,20 @@ chat_reset_size_request (gpointer widget)
 }
 
 static void
-chat_update_contacts_visibility (EmpathyChat *chat)
+chat_update_contacts_visibility (EmpathyChat *chat,
+			 gboolean show)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
-	gboolean show;
-
-	show = priv->remote_contact == NULL && priv->show_contacts;
+	GtkAllocation allocation;
 
 	if (!priv->scrolled_window_contacts) {
 		return;
 	}
 
+	if (priv->remote_contact != NULL) {
+		show = FALSE;
+	}
+
 	if (show && priv->contact_list_view == NULL) {
 		EmpathyContactListStore *store;
 		gint                     min_width;
@@ -1839,7 +1898,8 @@ chat_update_contacts_visibility (EmpathyChat *chat)
 		 * chat view is bigger the contact list will take some space on
 		 * it but we make sure the chat view don't become smaller than
 		 * 250. Relax the size request once the resize is done */
-		min_width = MIN (priv->vbox_left->allocation.width, 250);
+		gtk_widget_get_allocation (priv->vbox_left, &allocation);
+		min_width = MIN (allocation.width, 250);
 		gtk_widget_set_size_request (priv->vbox_left, min_width, -1);
 		g_idle_add (chat_reset_size_request, priv->vbox_left);
 
@@ -1878,7 +1938,7 @@ empathy_chat_set_show_contacts (EmpathyChat *chat,
 
 	priv->show_contacts = show;
 
-	chat_update_contacts_visibility (chat);
+	chat_update_contacts_visibility (chat, show);
 
 	g_object_notify (G_OBJECT (chat), "show-contacts");
 }
@@ -1908,7 +1968,7 @@ chat_remote_contact_changed_cb (EmpathyChat *chat)
 		g_object_get (channel, "handle-type", &priv->handle_type, NULL);
 	}
 
-	chat_update_contacts_visibility (chat);
+	chat_update_contacts_visibility (chat, priv->show_contacts);
 
 	g_object_notify (G_OBJECT (chat), "remote-contact");
 	g_object_notify (G_OBJECT (chat), "id");
@@ -1933,9 +1993,22 @@ chat_destroy_cb (EmpathyTpChat *tp_chat,
 
 	empathy_chat_view_append_event (chat->view, _("Disconnected"));
 	gtk_widget_set_sensitive (chat->input_text_view, FALSE);
-	empathy_chat_set_show_contacts (chat, FALSE);
+
+	chat_update_contacts_visibility (chat, FALSE);
 }
 
+static gboolean
+chat_hpaned_pos_changed_cb (GtkWidget* hpaned, gpointer user_data)
+{
+	gint hpaned_pos;
+	hpaned_pos = gtk_paned_get_position (GTK_PANED(hpaned));
+	empathy_conf_set_int (empathy_conf_get (),
+			      EMPATHY_PREFS_UI_CHAT_WINDOW_PANED_POS,
+			      hpaned_pos);
+	return TRUE;
+}
+
+
 static void
 show_pending_messages (EmpathyChat *chat) {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
@@ -1944,6 +2017,9 @@ show_pending_messages (EmpathyChat *chat) {
 	if (chat->view == NULL || priv->tp_chat == NULL)
 		return;
 
+	if (!priv->can_show_pending)
+		return;
+
 	messages = empathy_tp_chat_get_pending_messages (priv->tp_chat);
 
 	for (l = messages; l != NULL ; l = g_list_next (l)) {
@@ -1961,6 +2037,7 @@ chat_create_ui (EmpathyChat *chat)
  	GList           *list = NULL;
 	gchar           *filename;
 	GtkTextBuffer   *buffer;
+	gint              paned_pos;
 
 	filename = empathy_file_lookup ("empathy-chat.ui",
 					"libempathy-gtk");
@@ -1979,6 +2056,12 @@ chat_create_ui (EmpathyChat *chat)
 
 	/* Add message view. */
 	chat->view = empathy_theme_manager_create_view (empathy_theme_manager_get ());
+	/* If this is a GtkTextView, it's set as a drag destination for text/plain
+	   and other types, even though it's non-editable and doesn't accept any
+	   drags.  This steals drag motion for anything inside the scrollbars,
+	   making drag destinations on chat windows far less useful.
+	 */
+	gtk_drag_dest_unset (GTK_WIDGET (chat->view));
 	g_signal_connect (chat->view, "focus_in_event",
 			  G_CALLBACK (chat_text_view_focus_in_event_cb),
 			  chat);
@@ -2021,6 +2104,17 @@ chat_create_ui (EmpathyChat *chat)
 	/* Initialy hide the topic, will be shown if not empty */
 	gtk_widget_hide (priv->hbox_topic);
 
+	g_signal_connect (priv->hpaned, "notify::position",
+			  G_CALLBACK (chat_hpaned_pos_changed_cb),
+			  NULL);
+
+        /* Load the paned position */
+	if (empathy_conf_get_int (empathy_conf_get (),
+				 EMPATHY_PREFS_UI_CHAT_WINDOW_PANED_POS,
+				 &paned_pos)
+		&& paned_pos)
+		gtk_paned_set_position (GTK_PANED(priv->hpaned), paned_pos);
+
 	/* Set widget focus order */
 	list = g_list_append (NULL, priv->scrolled_window_input);
 	gtk_container_set_focus_chain (GTK_CONTAINER (priv->vbox_left), list);
@@ -2053,7 +2147,7 @@ chat_size_request (GtkWidget      *widget,
 
   child = gtk_bin_get_child (bin);
 
-  if (child && GTK_WIDGET_VISIBLE (child))
+  if (child && gtk_widget_get_visible (child))
     {
       GtkRequisition child_requisition;
 
@@ -2072,11 +2166,11 @@ chat_size_allocate (GtkWidget     *widget,
   GtkAllocation child_allocation;
   GtkWidget *child;
 
-  widget->allocation = *allocation;
+  gtk_widget_set_allocation (widget, allocation);
 
   child = gtk_bin_get_child (bin);
 
-  if (child && GTK_WIDGET_VISIBLE (child))
+  if (child && gtk_widget_get_visible (child))
     {
       child_allocation.x = allocation->x + gtk_container_get_border_width (GTK_CONTAINER (widget));
       child_allocation.y = allocation->y + gtk_container_get_border_width (GTK_CONTAINER (widget));
@@ -2150,8 +2244,11 @@ static void
 chat_constructed (GObject *object)
 {
 	EmpathyChat *chat = EMPATHY_CHAT (object);
+	EmpathyChatPriv *priv = GET_PRIV (chat);
 
-	chat_add_logs (chat);
+	if (priv->handle_type != TP_HANDLE_TYPE_ROOM)
+		chat_add_logs (chat);
+	priv->can_show_pending = TRUE;
 	show_pending_messages (chat);
 }
 
@@ -2534,6 +2631,9 @@ empathy_chat_set_tp_chat (EmpathyChat   *chat,
 	g_signal_connect (tp_chat, "members-changed",
 			  G_CALLBACK (chat_members_changed_cb),
 			  chat);
+	g_signal_connect (tp_chat, "member-renamed",
+			  G_CALLBACK (chat_member_renamed_cb),
+			  chat);
 	g_signal_connect_swapped (tp_chat, "notify::remote-contact",
 				  G_CALLBACK (chat_remote_contact_changed_cb),
 				  chat);
@@ -2770,3 +2870,23 @@ empathy_chat_is_room (EmpathyChat *chat)
 	return (priv->handle_type == TP_HANDLE_TYPE_ROOM);
 }
 
+guint
+empathy_chat_get_nb_unread_messages (EmpathyChat *self)
+{
+	EmpathyChatPriv *priv = GET_PRIV (self);
+
+	g_return_val_if_fail (EMPATHY_IS_CHAT (self), FALSE);
+
+	return priv->unread_messages;
+}
+
+/* called when the messages have been read by user */
+void
+empathy_chat_messages_read (EmpathyChat *self)
+{
+	EmpathyChatPriv *priv = GET_PRIV (self);
+
+	g_return_if_fail (EMPATHY_IS_CHAT (self));
+
+	priv->unread_messages = 0;
+}
