@@ -32,6 +32,7 @@
 #include <champlain-gtk/champlain-gtk.h>
 #endif
 
+#include <telepathy-glib/account.h>
 #include <telepathy-glib/util.h>
 
 #include <libempathy/empathy-tp-contact-factory.h>
@@ -40,7 +41,6 @@
 #include <libempathy/empathy-location.h>
 #include <libempathy/empathy-time.h>
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-account.h>
 
 #include "empathy-contact-widget.h"
 #include "empathy-account-chooser.h"
@@ -718,7 +718,7 @@ contact_widget_contact_setup (EmpathyContactWidget *information)
 static void
 contact_widget_contact_update (EmpathyContactWidget *information)
 {
-  EmpathyAccount *account = NULL;
+  TpAccount *account = NULL;
   const gchar *id = NULL;
 
   /* Connect and get info from new contact */
@@ -758,7 +758,7 @@ contact_widget_contact_update (EmpathyContactWidget *information)
         {
           const gchar *name;
 
-          name = empathy_account_get_display_name (account);
+          name = tp_account_get_display_name (account);
           gtk_label_set_label (GTK_LABEL (information->widget_account), name);
         }
     }
@@ -903,6 +903,7 @@ contact_widget_presence_notify_cb (EmpathyContactWidget *information)
   gtk_image_set_from_icon_name (GTK_IMAGE (information->image_state),
       empathy_icon_name_for_contact (information->contact),
       GTK_ICON_SIZE_BUTTON);
+  gtk_widget_show (information->image_state);
 }
 
 static void
@@ -1320,8 +1321,22 @@ contact_widget_location_update (EmpathyContactWidget *information)
   gboolean has_position = TRUE;
   GtkWidget *label;
   guint row = 0;
-  GHashTableIter iter;
-  gpointer key, pvalue;
+  static const gchar* ordered_geolocation_keys[] = {
+    EMPATHY_LOCATION_TEXT,
+    EMPATHY_LOCATION_URI,
+    EMPATHY_LOCATION_DESCRIPTION,
+    EMPATHY_LOCATION_BUILDING,
+    EMPATHY_LOCATION_FLOOR,
+    EMPATHY_LOCATION_ROOM,
+    EMPATHY_LOCATION_STREET,
+    EMPATHY_LOCATION_AREA,
+    EMPATHY_LOCATION_LOCALITY,
+    EMPATHY_LOCATION_REGION,
+    EMPATHY_LOCATION_COUNTRY,
+    NULL
+  };
+  int i;
+  const gchar *skey;
 
   if (!(information->flags & EMPATHY_CONTACT_WIDGET_SHOW_LOCATION))
     {
@@ -1379,18 +1394,18 @@ contact_widget_location_update (EmpathyContactWidget *information)
   gtk_box_pack_start (GTK_BOX (information->subvbox_location),
       information->table_location, FALSE, FALSE, 5);
 
-  g_hash_table_iter_init (&iter, location);
-  while (g_hash_table_iter_next (&iter, &key, &pvalue))
+
+  for (i = 0; (skey = ordered_geolocation_keys[i]); i++)
     {
-      const gchar *skey;
       const gchar* user_label;
       GValue *gvalue;
       char *svalue = NULL;
 
-      skey = (const gchar *) key;
+      gvalue = g_hash_table_lookup (location, (gpointer) skey);
+      if (gvalue == NULL)
+        continue;
 
       user_label = location_key_to_label (skey);
-      gvalue = (GValue *) pvalue;
 
       label = gtk_label_new (user_label);
       gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
@@ -1423,12 +1438,21 @@ contact_widget_location_update (EmpathyContactWidget *information)
               label, 1, 2, row, row + 1);
           gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
           gtk_widget_show (label);
+
+          if (!(information->flags & EMPATHY_CONTACT_WIDGET_FOR_TOOLTIP))
+            gtk_label_set_selectable (GTK_LABEL (label), TRUE);
         }
 
       g_free (svalue);
       row++;
     }
 
+  if (row == 0)
+    {
+      gtk_widget_hide (information->vbox_location);
+      return;
+    }
+
   gtk_widget_show (information->table_location);
 
 #if HAVE_LIBCHAMPLAIN
