@@ -97,8 +97,7 @@ account_assistant_build_error_page (EmpathyAccountAssistant *self,
     GError *error, gint page_num)
 {
   GtkWidget *main_vbox, *w, *hbox;
-  GString *str;
-  char *message;
+  const char *message;
   PangoAttrList *list;
   EmpathyAccountAssistantPriv *priv = GET_PRIV (self);
 
@@ -115,24 +114,16 @@ account_assistant_build_error_page (EmpathyAccountAssistant *self,
   gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
   gtk_widget_show (w);
 
-  /* translators: this is followed by the "while ..." strings some lines
-   * down this file.
-   */
-  str = g_string_new (_("There has been an error "));
-
   if (page_num == PAGE_IMPORT)
-    /* translators: this follows the "There has been an error " string */
-    str = g_string_append (str, _("while importing the accounts."));
+    message = _("There has been an error while importing the accounts.");
   else if (page_num >= PAGE_ENTER_CREATE &&
       priv->first_resp == RESPONSE_ENTER_ACCOUNT)
-    /* translators: this follows the "There has been an error " string */
-    str = g_string_append (str, _("while parsing the account details."));
+    message = _("There has been an error while parsing the account details.");
   else if (page_num >= PAGE_ENTER_CREATE &&
       priv->first_resp == RESPONSE_CREATE_ACCOUNT)
-    /* translators: this follows the "There has been an error " string */
-    str = g_string_append (str, _("while creating the account."));
-
-  message = g_string_free (str, FALSE);
+    message = _("There has been an error while creating the account.");
+  else
+    message = _("There has been an error.");
 
   w = gtk_label_new (message);
   gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
@@ -144,7 +135,6 @@ account_assistant_build_error_page (EmpathyAccountAssistant *self,
   gtk_label_set_line_wrap (GTK_LABEL (w), TRUE);
   gtk_widget_show (w);
 
-  g_free (message);
   pango_attr_list_unref (list);
 
   message = g_markup_printf_escaped
@@ -157,7 +147,7 @@ account_assistant_build_error_page (EmpathyAccountAssistant *self,
   gtk_widget_show (w);
 
   w = gtk_label_new (_("You can either go back and try to enter your "
-          "accounts' details again or quit this wizard and add accounts "
+          "accounts' details again or quit this assistant and add accounts "
           "later from the Edit menu."));
   gtk_box_pack_start (GTK_BOX (main_vbox), w, FALSE, FALSE, 6);
   gtk_misc_set_alignment (GTK_MISC (w), 0, 0.5);
@@ -235,7 +225,7 @@ account_assistant_account_enabled_cb (GObject *source,
   EmpathyAccountAssistant *self = user_data;
   EmpathyAccountAssistantPriv *priv = GET_PRIV (self);
 
-  empathy_account_set_enabled_finish (EMPATHY_ACCOUNT (source),
+  tp_account_set_enabled_finish (TP_ACCOUNT (source),
       result, &error);
 
   if (error)
@@ -259,10 +249,20 @@ account_assistant_apply_account_cb (GObject *source,
   EmpathyAccountAssistant *self = user_data;
   EmpathyAccountAssistantPriv *priv = GET_PRIV (self);
   EmpathyAccountSettings *settings = EMPATHY_ACCOUNT_SETTINGS (source);
-  EmpathyAccount *account;
+  TpAccount *account;
+  gchar *display_name;
 
   empathy_account_settings_apply_finish (settings, result, &error);
 
+  /* set default display name */
+  display_name = empathy_account_widget_get_default_display_name (
+      priv->current_widget_object);
+
+  empathy_account_settings_set_display_name_async (settings,
+      display_name, NULL, NULL);
+
+  g_free (display_name);
+
   priv->is_creating = FALSE;
 
   if (error != NULL)
@@ -275,7 +275,7 @@ account_assistant_apply_account_cb (GObject *source,
 
   /* enable the newly created account */
   account = empathy_account_settings_get_account (settings);
-  empathy_account_set_enabled_async (account, TRUE,
+  tp_account_set_enabled_async (account, TRUE,
       account_assistant_account_enabled_cb, self);
 }
 
@@ -315,11 +315,12 @@ account_assistant_protocol_changed_cb (GtkComboBox *chooser,
   char *str;
   GtkWidget *account_widget;
   EmpathyAccountWidget *widget_object = NULL;
+  gboolean is_gtalk;
 
   priv = GET_PRIV (self);
 
   cm = empathy_protocol_chooser_dup_selected (
-      EMPATHY_PROTOCOL_CHOOSER (chooser), &proto);
+      EMPATHY_PROTOCOL_CHOOSER (chooser), &proto, &is_gtalk);
 
   if (cm == NULL || proto == NULL)
     /* we are not ready yet */
@@ -327,15 +328,20 @@ account_assistant_protocol_changed_cb (GtkComboBox *chooser,
 
   /* Create account */
   /* To translator: %s is the protocol name */
-  str = g_strdup_printf (_("New %s account"), proto->name);
+  str = g_strdup_printf (_("New %s account"),
+      empathy_protocol_name_to_display_name (
+          is_gtalk ? "gtalk" : proto->name));
 
   settings = empathy_account_settings_new (cm->name, proto->name, str);
 
+  if (is_gtalk)
+    empathy_account_settings_set_icon_name_async (settings, "im-google-talk",
+      NULL, NULL);
+
   if (priv->first_resp == RESPONSE_CREATE_ACCOUNT)
     empathy_account_settings_set_boolean (settings, "register", TRUE);
 
-  widget_object = empathy_account_widget_new_for_protocol (proto->name,
-      settings, TRUE);
+  widget_object = empathy_account_widget_new_for_protocol (settings, TRUE);
   account_widget = empathy_account_widget_get_widget (widget_object);
 
   if (priv->current_account_widget != NULL)
@@ -934,8 +940,20 @@ empathy_account_assistant_init (EmpathyAccountAssistant *self)
 }
 
 GtkWidget *
-empathy_account_assistant_new (GtkWindow *window)
+empathy_account_assistant_show (GtkWindow *window)
 {
-  return g_object_new (EMPATHY_TYPE_ACCOUNT_ASSISTANT, "parent-window",
-      window, NULL);
+  static GtkWidget *dialog = NULL;
+
+  if (dialog == NULL)
+    {
+      dialog =  g_object_new (EMPATHY_TYPE_ACCOUNT_ASSISTANT, "parent-window",
+        window, NULL);
+      g_object_add_weak_pointer (G_OBJECT (dialog), (gpointer *) &dialog);
+    }
+
+  gtk_window_present (GTK_WINDOW (dialog));
+
+  return dialog;
 }
+
+
