@@ -46,6 +46,7 @@
 #include <libempathy-gtk/empathy-geometry.h>
 #include <libempathy-gtk/empathy-gtk-enum-types.h>
 #include <libempathy-gtk/empathy-new-message-dialog.h>
+#include <libempathy-gtk/empathy-new-call-dialog.h>
 #include <libempathy-gtk/empathy-log-window.h>
 #include <libempathy-gtk/empathy-presence-chooser.h>
 #include <libempathy-gtk/empathy-sound.h>
@@ -94,6 +95,7 @@ typedef struct {
 	GtkWidget              *presence_chooser;
 	GtkWidget              *errors_vbox;
 
+	GtkToggleAction        *show_protocols;
 	GtkRadioAction         *sort_by_name;
 	GtkRadioAction         *sort_by_status;
 	GtkRadioAction         *normal_with_avatars;
@@ -131,8 +133,9 @@ main_window_flash_stop (EmpathyMainWindow *window)
 }
 
 typedef struct {
-	EmpathyEvent *event;
-	gboolean      on;
+	EmpathyEvent       *event;
+	gboolean            on;
+	EmpathyMainWindow  *window;
 } FlashForeachData;
 
 static gboolean
@@ -146,6 +149,7 @@ main_window_flash_foreach (GtkTreeModel *model,
 	const gchar      *icon_name;
 	GtkTreePath      *parent_path = NULL;
 	GtkTreeIter       parent_iter;
+	GdkPixbuf        *pixbuf = NULL;
 
 	/* To be used with gtk_tree_model_foreach, update the status icon
 	 * of the contact to show the event icon (on=TRUE) or the presence
@@ -163,12 +167,15 @@ main_window_flash_foreach (GtkTreeModel *model,
 
 	if (data->on) {
 		icon_name = data->event->icon_name;
+		pixbuf = empathy_pixbuf_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
 	} else {
-		icon_name = empathy_icon_name_for_contact (contact);
+		pixbuf = contact_list_store_get_contact_status_icon (
+						data->window->list_store,
+						contact);
 	}
 
 	gtk_tree_store_set (GTK_TREE_STORE (model), iter,
-			    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, icon_name,
+			    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, pixbuf,
 			    -1);
 
 	/* To make sure the parent is shown correctly, we emit
@@ -203,6 +210,7 @@ main_window_flash_cb (EmpathyMainWindow *window)
 	events = empathy_event_manager_get_events (window->event_manager);
 	for (l = events; l; l = l->next) {
 		data.event = l->data;
+		data.window = window;
 		if (!data.event->contact || !data.event->must_ack) {
 			continue;
 		}
@@ -257,6 +265,7 @@ main_window_event_removed_cb (EmpathyEventManager *manager,
 
 	data.on = FALSE;
 	data.event = event;
+	data.window = window;
 	gtk_tree_model_foreach (GTK_TREE_MODEL (window->list_store),
 				main_window_flash_foreach,
 				&data);
@@ -303,19 +312,37 @@ main_window_row_activated_cb (EmpathyContactListView *view,
 	g_object_unref (contact);
 }
 
+static void
+main_window_remove_error (EmpathyMainWindow *window,
+			  TpAccount *account)
+{
+	GtkWidget *error_widget;
+
+	error_widget = g_hash_table_lookup (window->errors, account);
+	if (error_widget != NULL) {
+		gtk_widget_destroy (error_widget);
+		g_hash_table_remove (window->errors, account);
+	}
+}
+
+static void
+main_window_account_disabled_cb (TpAccountManager *manager,
+				 TpAccount *account,
+				 EmpathyMainWindow *window)
+{
+	main_window_remove_error (window, account);
+}
+
 static void
 main_window_error_retry_clicked_cb (GtkButton *button,
-				   EmpathyMainWindow *window)
+				    EmpathyMainWindow *window)
 {
 	TpAccount *account;
-	GtkWidget *error_widget;
 
 	account = g_object_get_data (G_OBJECT (button), "account");
 	tp_account_reconnect_async (account, NULL, NULL);
 
-	error_widget = g_hash_table_lookup (window->errors, account);
-	gtk_widget_destroy (error_widget);
-	g_hash_table_remove (window->errors, account);
+	main_window_remove_error (window, account);
 }
 
 static void
@@ -323,14 +350,11 @@ main_window_error_edit_clicked_cb (GtkButton *button,
 				   EmpathyMainWindow *window)
 {
 	TpAccount *account;
-	GtkWidget *error_widget;
 
 	account = g_object_get_data (G_OBJECT (button), "account");
 	empathy_accounts_dialog_show (GTK_WINDOW (window->window), account);
 
-	error_widget = g_hash_table_lookup (window->errors, account);
-	gtk_widget_destroy (error_widget);
-	g_hash_table_remove (window->errors, account);
+	main_window_remove_error (window, account);
 }
 
 static void
@@ -338,12 +362,9 @@ main_window_error_close_clicked_cb (GtkButton *button,
 				    EmpathyMainWindow *window)
 {
 	TpAccount *account;
-	GtkWidget *error_widget;
 
 	account = g_object_get_data (G_OBJECT (button), "account");
-	error_widget = g_hash_table_lookup (window->errors, account);
-	gtk_widget_destroy (error_widget);
-	g_hash_table_remove (window->errors, account);
+	main_window_remove_error (window, account);
 }
 
 static void
@@ -507,50 +528,7 @@ main_window_connection_changed_cb (TpAccount  *account,
 	    reason != TP_CONNECTION_STATUS_REASON_REQUESTED) {
 		const gchar *message;
 
-		switch (reason) {
-		case TP_CONNECTION_STATUS_REASON_NONE_SPECIFIED:
-			message = _("No error specified");
-			break;
-		case TP_CONNECTION_STATUS_REASON_NETWORK_ERROR:
-			message = _("Network error");
-			break;
-		case TP_CONNECTION_STATUS_REASON_AUTHENTICATION_FAILED:
-			message = _("Authentication failed");
-			break;
-		case TP_CONNECTION_STATUS_REASON_ENCRYPTION_ERROR:
-			message = _("Encryption error");
-			break;
-		case TP_CONNECTION_STATUS_REASON_NAME_IN_USE:
-			message = _("Name in use");
-			break;
-		case TP_CONNECTION_STATUS_REASON_CERT_NOT_PROVIDED:
-			message = _("Certificate not provided");
-			break;
-		case TP_CONNECTION_STATUS_REASON_CERT_UNTRUSTED:
-			message = _("Certificate untrusted");
-			break;
-		case TP_CONNECTION_STATUS_REASON_CERT_EXPIRED:
-			message = _("Certificate expired");
-			break;
-		case TP_CONNECTION_STATUS_REASON_CERT_NOT_ACTIVATED:
-			message = _("Certificate not activated");
-			break;
-		case TP_CONNECTION_STATUS_REASON_CERT_HOSTNAME_MISMATCH:
-			message = _("Certificate hostname mismatch");
-			break;
-		case TP_CONNECTION_STATUS_REASON_CERT_FINGERPRINT_MISMATCH:
-			message = _("Certificate fingerprint mismatch");
-			break;
-		case TP_CONNECTION_STATUS_REASON_CERT_SELF_SIGNED:
-			message = _("Certificate self-signed");
-			break;
-		case TP_CONNECTION_STATUS_REASON_CERT_OTHER_ERROR:
-			message = _("Certificate error");
-			break;
-		default:
-			message = _("Unknown error");
-			break;
-		}
+		message = empathy_status_reason_get_default_message (reason);
 
 		main_window_error_display (window, account, message);
 	}
@@ -561,17 +539,11 @@ main_window_connection_changed_cb (TpAccount  *account,
 	}
 
 	if (current == TP_CONNECTION_STATUS_CONNECTED) {
-		GtkWidget *error_widget;
-
 		empathy_sound_play (GTK_WIDGET (window->window),
 				    EMPATHY_SOUND_ACCOUNT_CONNECTED);
 
 		/* Account connected without error, remove error message if any */
-		error_widget = g_hash_table_lookup (window->errors, account);
-		if (error_widget) {
-			gtk_widget_destroy (error_widget);
-			g_hash_table_remove (window->errors, account);
-		}
+		main_window_remove_error (window, account);
 	}
 }
 
@@ -695,6 +667,13 @@ main_window_chat_new_message_cb (GtkAction         *action,
 	empathy_new_message_dialog_show (GTK_WINDOW (window->window));
 }
 
+static void
+main_window_chat_new_call_cb (GtkAction         *action,
+				 EmpathyMainWindow *window)
+{
+	empathy_new_call_dialog_show (GTK_WINDOW (window->window));
+}
+
 static void
 main_window_chat_add_contact_cb (GtkAction         *action,
 				 EmpathyMainWindow *window)
@@ -784,6 +763,21 @@ main_window_view_sort_contacts_cb (GtkRadioAction    *action,
 	empathy_contact_list_store_set_sort_criterium (window->list_store, value);
 }
 
+static void
+main_window_view_show_protocols_cb (GtkToggleAction *action,
+					EmpathyMainWindow *window)
+{
+	gboolean value;
+
+	value = gtk_toggle_action_get_active (action);
+
+	empathy_conf_set_bool (empathy_conf_get (),
+					 EMPATHY_PREFS_UI_SHOW_PROTOCOLS,
+					 value == TRUE);
+	empathy_contact_list_store_set_show_protocols (window->list_store,
+					 value == TRUE);
+}
+
 /* Matches GtkRadioAction values set in empathy-main-window.ui */
 #define CONTACT_LIST_NORMAL_SIZE_WITH_AVATARS		0
 #define CONTACT_LIST_NORMAL_SIZE			1
@@ -809,8 +803,26 @@ main_window_view_contacts_list_size_cb (GtkRadioAction    *action,
 						     value == CONTACT_LIST_NORMAL_SIZE_WITH_AVATARS);
 	empathy_contact_list_store_set_is_compact (window->list_store,
 						   value == CONTACT_LIST_COMPACT_SIZE);
+
+	gtk_action_set_sensitive (GTK_ACTION (window->show_protocols),
+						value != CONTACT_LIST_COMPACT_SIZE );
 }
 
+static void main_window_notify_show_protocols_cb (EmpathyConf       *conf,
+					const gchar       *key,
+					EmpathyMainWindow *window)
+{
+	gboolean show_protocols;
+
+	if (empathy_conf_get_bool (conf,
+				   EMPATHY_PREFS_UI_SHOW_PROTOCOLS,
+				   &show_protocols)) {
+		gtk_toggle_action_set_active (window->show_protocols,
+					      show_protocols);
+	}
+}
+
+
 static void
 main_window_notify_contact_list_size_cb (EmpathyConf       *conf,
 					 const gchar       *key,
@@ -1109,6 +1121,9 @@ main_window_account_removed_cb (TpAccountManager  *manager,
 		g_list_length (a) > 0);
 
 	g_list_free (a);
+
+	/* remove errors if any */
+	main_window_remove_error (window, account);
 }
 
 static void
@@ -1142,6 +1157,7 @@ main_window_connection_items_setup (EmpathyMainWindow *window,
 	const gchar *actions_connected[] = {
 		"room",
 		"chat_new_message",
+		"chat_new_call",
 		"chat_add_contact",
 		"edit_personal_information"
 	};
@@ -1194,21 +1210,6 @@ account_manager_prepared_cb (GObject *source_object,
 	g_list_free (accounts);
 }
 
-static gboolean
-main_window_state_event_cb (GtkWidget           *widget,
-			    GdkEventWindowState *event,
-			    EmpathyMainWindow   *window)
-{
-	/* Force reloading geometry because some WM lose it in the
-	 * iconify/deiconify process */
-	if ((event->changed_mask & GDK_WINDOW_STATE_ICONIFIED) != 0 &&
-	    (event->new_window_state & GDK_WINDOW_STATE_ICONIFIED) == 0) {
-		empathy_geometry_load (GTK_WINDOW (window->window), GEOMETRY_NAME);
-	}
-
-	return FALSE;
-}
-
 GtkWidget *
 empathy_main_window_show (void)
 {
@@ -1242,6 +1243,7 @@ empathy_main_window_show (void)
 				       "errors_vbox", &window->errors_vbox,
 				       "ui_manager", &window->ui_manager,
 				       "view_show_offline", &show_offline_widget,
+				       "view_show_protocols", &window->show_protocols,
 				       "view_sort_by_name", &window->sort_by_name,
 				       "view_sort_by_status", &window->sort_by_status,
 				       "view_normal_size_with_avatars", &window->normal_with_avatars,
@@ -1257,9 +1259,9 @@ empathy_main_window_show (void)
 
 	empathy_builder_connect (gui, window,
 			      "main_window", "destroy", main_window_destroy_cb,
-			      "main_window", "window-state-event", main_window_state_event_cb,
 			      "chat_quit", "activate", main_window_chat_quit_cb,
 			      "chat_new_message", "activate", main_window_chat_new_message_cb,
+			      "chat_new_call", "activate", main_window_chat_new_call_cb,
 			      "view_history", "activate", main_window_view_history_cb,
 			      "room_join_new", "activate", main_window_room_join_new_cb,
 			      "room_join_favorites", "activate", main_window_room_join_favorites_cb,
@@ -1267,6 +1269,7 @@ empathy_main_window_show (void)
 			      "chat_add_contact", "activate", main_window_chat_add_contact_cb,
 			      "view_show_ft_manager", "activate", main_window_view_show_ft_manager,
 			      "view_show_offline", "toggled", main_window_view_show_offline_cb,
+			      "view_show_protocols", "toggled", main_window_view_show_protocols_cb,
 			      "view_sort_by_name", "changed", main_window_view_sort_contacts_cb,
 			      "view_normal_size_with_avatars", "changed", main_window_view_contacts_list_size_cb,
 			      "view_show_map", "activate", main_window_view_show_map_cb,
@@ -1380,6 +1383,9 @@ empathy_main_window_show (void)
 	g_signal_connect (window->account_manager, "account-removed",
 			  G_CALLBACK (main_window_account_removed_cb),
 			  window);
+	g_signal_connect (window->account_manager, "account-disabled",
+			  G_CALLBACK (main_window_account_disabled_cb),
+			  window);
 
 	l = empathy_event_manager_get_events (window->event_manager);
 	while (l) {
@@ -1401,6 +1407,16 @@ empathy_main_window_show (void)
 
 	gtk_toggle_action_set_active (show_offline_widget, show_offline);
 
+	/* Show protocol ? */
+	empathy_conf_notify_add (conf,
+				 EMPATHY_PREFS_UI_SHOW_PROTOCOLS,
+				 (EmpathyConfNotifyFunc) main_window_notify_show_protocols_cb,
+				 window);
+
+	main_window_notify_show_protocols_cb (conf,
+					    EMPATHY_PREFS_UI_SHOW_PROTOCOLS,
+					    window);
+
 	/* Sort by name / by status ? */
 	empathy_conf_notify_add (conf,
 				 EMPATHY_PREFS_CONTACTS_SORT_CRITERIUM,
